# ****************       BEGIN-STANDARD-COPYRIGHT      ***************
#
# Copyright (c) 2017, Spirent Communications.
#
# All rights reserved. Proprietary and confidential information of Spirent Communications.
#
#  ***************        END-STANDARD-COPYRIGHT       ***************

"""Topology class and necessary tools."""

from .session_profile import SessionProfile
from .identity import UriIdentity



class Topology(UriIdentity):
    """Topology representation.

    Provide a list of devices with sessions
    """

    def __init__(self, project, uri, devices, protocol):
        """Initiates a new topology.

        Arguments:
        protocol_socket -- an instance of ProtocolSocket. It must be connected by the time open() is called.
        agent_type -- agent type (local/itest/velocity)
        uri -- topology URI, e.g. project://my_project/topologies/ServerAndPC.tbml
        devices -- a list of DeviceData objects
        """
        UriIdentity.__init__(self, uri)
        self._project = project
        self._devices = dict((dev.name, Device(self, dev.uri, dev.sessions, protocol)) for dev in devices)

    def agent_type(self):
        """ Agent type"""
        return self._project.agent_type

    def list(self):
        """Return a list of devices available"""
        return list(self._devices.keys())

    def __dir__(self):
        """ return a list of methods available to project"""
        res = super.__dir__(self) + self.list()
        return res

    def __getitem__(self, key):
        """ Return a device by name"""
        return self._devices.get(key)

    def __getattr__(self, key):
        """ Return a device by name"""
        ret = self[key]
        if ret == None:
            raise AttributeError(key)
        return ret

class DeviceData(object):
    """ A device details """
    def __init__(self, name, uri, sessions):
        self.name = name
        self.uri = uri
        self.sessions = sessions

class Device(UriIdentity):
    """ A device in topology"""

    def __init__(self, topology, name, sessions, protocol):
        """ Initialize a device reference handle contained in topology """
        UriIdentity.__init__(self, topology.uri + '#' + name)
        self._topology = topology
        self._name = name
        self._protocol = protocol
        self._sessions = dict((session, self._create_session(session)) for session in sessions)

    def _create_session(self, session):
        session_uri =  self.uri + ':' + session
        return SessionProfile(self._protocol, self.agent_type(), session_uri)

    def agent_type(self):
        """ A type of agent"""
        return self._topology.agent_type()

    def name(self):
        """ A name of device """
        return self._name

    def list(self):
        """ A list of available sessions"""
        return list(self._sessions.keys())

    def __dir__(self):
        """ return a list of methods available to project"""
        res = super.__dir__(self) + self.list()
        return res

    def __getitem__(self, key):
        """ Return a device session by name"""
        return self._sessions.get(key)

    def __getattr__(self, key):
        """ Return a device session by name"""
        ret = self[key]
        if ret is None:
            raise AttributeError(key)
        return ret



# ****************       BEGIN-STANDARD-COPYRIGHT      ***************
#
# Copyright (c) 2017, Spirent Communications.
#
# All rights reserved. Proprietary and confidential information of Spirent Communications.
#
#  ***************        END-STANDARD-COPYRIGHT       ***************

"""Main SLC library API entry point."""

import os
import shutil
import tempfile
import six

from sys import platform as _platform
from . import config as cfg
from .internal.protocol import ProtocolSocket
from .project import Project
from .builtin_sessions import BuiltinSession
from .python_name import python_name

try:
    import subprocess
    from subprocess import DEVNULL # Python 3.x
except ImportError:
    DEVNULL = open(os.devnull, 'wb')

_DEBUG_SLC = False

try:
    from urlparse import urlparse
except ImportError:
    # in Python 3, urlparse renamed to urllib.parse
    # pylint: disable=no-name-in-module,import-error
    from urllib.parse import urlparse

# pylint: disable=invalid-name
_SLC_instance = None

class SLCError(Exception):
    """General exception indicating various types of SLC errors."""

    pass

class _SLC(object):
    """A private class to represent SLC.

    Objects of this class should not be created directly, but rather obtained via the init() function below.
    """

    def __init__(self, isLocal, host, port, agent_path, itar_path, license_server, license_proxy):
        global _SLC_instance

        if _SLC_instance:
            raise SLCError('Multiple instances of SLC are not supported')

        self._agent_path = agent_path

        self.agent_log_file = None

        if isLocal:
            if agent_path.endswith('/') or agent_path.endswith('\\'):
                # remove last '/' from path
                agent_path = agent_path[:-1]

            if _platform == "darwin":
                # we need to check path is correct for macos
                if os.path.isdir(agent_path) and not agent_path.endswith('/Contents/Eclipse'):
                    agent_path += '/Contents/Eclipse/'

            agent_jre = agent_path + os.sep + 'jre'

            java_bin = None
            self.temp_dir =  tempfile.mkdtemp('velocity_agent')
            self.agent_data_dir = self.temp_dir + os.sep + '@no'
            self.agent_log_file_name = self.temp_dir + os.sep + 'agent.log'
            self.agent_log_file = open(self.agent_log_file_name, 'w')

            tmp_param = '-Djava.io.tmpdir=' + tempfile.gettempdir()
            if _platform == "linux" or _platform == "linux2":
                # linux
                java_bin = agent_jre +'/bin/java'
            elif _platform == "darwin":
                # MAC OS X
                java_bin = agent_jre +'/Contents/Home/jre/bin/java'
            elif _platform == "win32":
                # Windows
                java_bin = agent_jre +'\\bin\\java.exe'
            elif _platform == "win64":
                # Windows 64-bit
                java_bin = agent_jre +'\\bin\\java.exe'

            if not os.path.isfile(java_bin):
                java_bin = 'java'

            if os.path.isfile(agent_path):
                agent_path = os.path.dirname(agent_path)
            if not os.path.isdir(agent_path):
                raise SLCError('Agent is not found at: ' + agent_path + "\nAborting... Please verify: SPIRENT_SLC_AGENT_PATH environemnt variable")

            self.agent_args = [java_bin,
                          '-Xmx512M',
                          '-Dfile.encoding=UTF-8',
                          '-XX:MaxMetaspaceSize=128M',
                          '-Dorg.eclipse.emf.ecore.plugin.EcorePlugin.doNotLoadResourcesPlugin=true',
                          '-Djava.awt.headless=true',
                          tmp_param,
                          '-jar', agent_path + os.sep + 'plugins' + os.sep +
                            'org.eclipse.equinox.launcher_1.4.0.v20161219-1356.jar',
                          '-consoleLog', '-data', self.agent_data_dir]
            self.agent_extra_args = [   '--agentVelocityHost', host,
                                        '--sfAgentServerPort', str(port),
                                        '--listeningMode', 'library',
                                        '--sfAgentDisableSslValidation']


            if license_server != None:
                self.agent_extra_args.append('--licenseServer')
                self.agent_extra_args.append(license_server)
            else:
                raise SLCError('Please specify a valid SPIRENT_SLC_LICENSE_SERVER environment variable value in format host[:port]\n' +
                'And SPIRENT_SLC_LICENSE_PROXY=proxy_host[:proxy_port] in case x86_64 Velocity Agent is used...')

            if license_proxy != None:
                self.agent_extra_args.append('--licenseProxy')
                self.agent_extra_args.append(license_proxy)

            if itar_path != None:
                # print('add itar_path:', itar_path)
                self.agent_extra_args.append('--itar')
                self.agent_extra_args.append(itar_path)

            if cfg.is_verbose_mode():
                print('--->Agent verbose output\n')
                print("JAVA_BIN: " + str(java_bin))
                print("Starting velocity agent:\nPATH:" + str(agent_path))
                print("LOG FILE: " + str(self.agent_log_file.name))
                print("ITARPATH:" + str(itar_path))
                print("ARGUMENTS:\n\t", '\n\t'.join(self.agent_extra_args))
                print('--->')

            final_args = self.agent_args + self.agent_extra_args
            self.agent_log_file.write('Starting java with arguments:\n' + ' '.join(final_args) + '\n\n\n')
            self.agent_log_file.flush()

            self._agent_process = subprocess.Popen( final_args, stdin=None, stderr=subprocess.STDOUT, stdout=self.agent_log_file, shell=False)
            self._agent_type = "local"
        else:
            self._agent_process = None

            # agent type will be updated based on information retrieved from agent itself.
            self._agent_type = 'remote'
        self._protosock = None
        try:
            self._protosock = ProtocolSocket(host, port, cfg.get_agent_start_timeout(), self._agent_process)
        except:
            if self.agent_log_file != None:
                # Be sure we try terminate started agent

                print('\n\nERROR:\nFailure during Velocity Agent startup. Please refer to a log file created:\n\'' + self.agent_log_file_name + '\'')
                self.agent_log_file.close()

                ## Read and display last 50 lines of log.

                with open(self.agent_log_file_name, 'r') as file:
                    lines = file.readlines()
                    if len(lines) > 50:
                        lines = lines[-50:]

                    msg = '\n'.join(lines)
                    if 'Please specify --licenseProxy host[:port]' in msg:
                        print('Agent requires specification of SPIRENT_SLC_LICENSE_PROXY environment variable. Please refers to documentation.')
                    print('\nLast lines of Velocoty Agent log:\n',msg)

                self.agent_log_file = None
                # Also terminate velocity agent if error is happed during connection
                self.close()
            raise

        self._projects = dict()

        self._sync_projects()

        self.sessions = BuiltinSession(self._protosock, self._agent_type)

        _SLC_instance = self

    def _sync_projects(self):
        processed = []
        for project in self._protosock.list_projects():
            # TODO: All spaces in the name of a project or any other characters that are not legal in a Python identifier should be replaced by underscores
            # TODO: duplicates after replacing
            py_project_name = python_name(project)
            processed.append(py_project_name)
            if not py_project_name in self._projects.keys():
                self._projects[py_project_name] = Project(project, self._protosock, self._agent_type)

        # Remove all projects not available anymore
        for project in list(self._projects.keys()):
            if not project in processed:
                del self._projects[project]


    def list(self):
        """Returns a list of names of all projects."""
        self._sync_projects()
        return self._projects.keys()

    def __dir__(self):
        """ return a list of methods available to project"""
        res = super.__dir__(self) + list(self.list())
        return res

    def open(self, projectName):
        """Opens a project.

        Arguments:
        projectName -- project name.

        Returns the project. If project name is None, empty or not present in the projects' list, raises ValueError.
        """
        if not projectName in self._projects:
            raise SLCError('Faile to find project with name: ' + projectName)
        return self._projects[projectName].open()

    def __getitem__(self, key):
        """
            Allow to return a project by name and call open() over it.
        """
        if self._protosock is None:
            return None

        prj = self._projects.get(key)
        if prj:
            return prj

        return None

    def __getattr__(self, key):
        """
            Allow to return a project by name and call open() over it.
        """
        ret = self[key]
        if ret == None:
            raise AttributeError(key)
        return ret

    def close(self):
        """Closes SLC.

        Underlying connections are dropped (if any) and processes are terminated (if any).
        """
        global _SLC_instance
        _SLC_instance = None

        protosock = self._protosock
        self._protosock = None
        if protosock:
            protosock.close()

        p = self._agent_process
        self._agent_process = None
        if p:
            p.kill()
            if six.PY2:
                p.wait()
            else:
                p.wait(timeout=30)

        # remove Agent logs
        if self.agent_log_file != None:
            self.agent_log_file.close()
            os.remove(self.agent_log_file_name)
            shutil.rmtree(self.temp_dir)

    def __enter__(self):
        """ SLC could be used with 'with' as section. """
        return self

    def __exit__(self, *args):
        """ Automatically close then leave with section. """
        self.close()


    def is_open(self):
        """ Check if SLC connection is alive"""
        return self._protosock != None and self._protosock.is_open()

def _parse_host_port(host_port):
    """Parse a "host:port" string and returns (host, port) pair

    Arguments:
    host_port -- a string of the "host:port" format

    Return (host, port) pair. If the given string does not comply with the pattern "host:port", one or both components
    of the pair returned may be None.
    """

    url = urlparse('http://' + host_port)       # we need some schema for urlparse to work
    return (url.hostname, url.port)

def init(host=None, agent_path=None, itar_path=None, license_server=None, license_proxy=None, password=None):
    """Initialises and returns an SLC object.

    Arguments:
    host -- SLC host to connect to in a form of 'host:port' string. Env var: SPIRENT_SLC_HOST. Default: 'local', which
            means the library is to run its own instance of the Velocity agent.

    password -- password to connect to SLC. Env var: SPIRENT_SLC_PASSWORD. Default: no password.

    agent_path -- path to the 'velocity-agent' executable. Env var: SPIRENT_SLC_AGENT_PATH.
                  Default: find 'velocity-agent' in PATH.
                  If the agent is not found, and 'host' specifies 'local', SLCError is raised.
    itar_path -- path to location of local itar's. will use ITAR_PATH environment variable if not specified.
                In case of relative path, current working directory will be used to construct absolute path.

    license_server -- a host[:port] of license server.

    license_proxy -- a host[:port] of license server proxy.

    All arguments are optional. If something is not specified, the value is taken from the respective environment
    variable. If the respective environment variable is not set, the respective default value is used.

    Returns an SLC object.

    Only one SLC object (or none at all) may exist at a time. First time this function is called, it is created.
    After that, unless the object is closed via it's close() method, the same object is returned.
    If the returned object gets closed, the next init() call will create a new one.
    """

    if _SLC_instance:
        return _SLC_instance

    if not host:
        host = cfg.get_agent_host_port()

    isLocal = host == 'local'
    if isLocal:
        host = cfg.get_local_agent_host_port()

    host, port = _parse_host_port(host)

    if not agent_path:
        agent_path = cfg.get_agent_path()
    if not agent_path and isLocal:
        raise SLCError('agentPath is not specified and SPIRENT_SLC_AGENT_PATH environment variable is not set')

    if itar_path is None:
        itar_path = cfg.get_itar_path()
    else:
        # Check if it is relative path we need to add current script directory
        if not os.path.isabs(itar_path):
            itar_path = os.getcwd() + os.sep + itar_path

    if license_server is None:
        license_server = cfg.get_license_server()

    if license_proxy is None:
        license_proxy = cfg.get_license_proxy()

    return _SLC(isLocal, host, port, agent_path, itar_path, license_server, license_proxy)



# ****************       BEGIN-STANDARD-COPYRIGHT      ***************
#
# Copyright (c) 2017, Spirent Communications.
#
# All rights reserved. Proprietary and confidential information of Spirent Communications.
#
#  ***************        END-STANDARD-COPYRIGHT       ***************

"""Builtin sessions provider."""

from .session_profile import SessionProfile

class BuiltinSession(object):
    """Provide an access to builtin sessions"""

    def __init__(self, protocol_socket, agent_type):
        self._protosock = protocol_socket
        self._agent_type = agent_type
      
    @property
    def rft(self):
        """Return session profile obj with URI=application://com.fanfare.itest.appstore.applications.rft 
        
           The session window for iTest RFT sessions has been designed in partnership with IBM to enable you to launch Rational
           Functional Tester scripts from iTest. This integration is certified by IBM as Ready for IBM Rational Software.
        """
        return SessionProfile(self._protosock, self._agent_type, 'application://com.fanfare.itest.appstore.applications.rft')

    @property
    def cmd(self):
        """Return session profile obj with URI=application://com.fnfr.svt.applications.cmd 
        
           You can run Microsoft Windows Command Prompt sessions (that is, run cmd terminal sessions). For each open session, 
           the Command Prompt Session window displays your commands and the local PC's responses.
        """
        return SessionProfile(self._protosock, self._agent_type, 'application://com.fnfr.svt.applications.cmd')
    
    @property
    def chat(self):
        """Return session profile obj with URI=application://com.fnfr.bobcat.tools.chat
           
           You can add steps that receive and send XMPP chat messages during execution. A test case can send and
           receive as many messages as are needed. Message text can contain both fixed text and response data
        """
        return SessionProfile(self._protosock, self._agent_type, 'application://com.fnfr.bobcat.tools.chat')
    
    @property
    def serial(self):
        """Return session profile obj with URI=application://com.fnfr.itest.application.serial 
        
          For Serial Port sessions, the computer running iTest communicates directly over a serial port connection with the device
          under test. For each open session, the Serial Port session window displays your commands and the device's responses. You 
          can think of the session window as a terminal client - a terminal that iTest is monitoring and capturing.        
        """
        return SessionProfile(self._protosock, self._agent_type, 'application://com.fnfr.itest.application.serial')
    
    @property
    def tool(self):
        """Return session profile obj with URI=application://com.fnfr.itest.application.sf.tool """
        return SessionProfile(self._protosock, self._agent_type, 'application://com.fnfr.itest.application.sf.tool')
    
    @property
    def file(self):
        """Return session profile obj with URI=application://com.fnfr.itest.applications.file 
        
           The File session type enables an automated test case to work with text files during execution (open a 
           file, go to a specified position in the file, read a line, write a line, and so on).
        """
        return SessionProfile(self._protosock, self._agent_type, 'application://com.fnfr.itest.applications.file')
    
    @property
    def landslide(self):
        """Return session profile obj with URI=application://com.fnfr.itest.applications.landslide 
        """
        return SessionProfile(self._protosock, self._agent_type, 'application://com.fnfr.itest.applications.landslide')
    
    @property
    def ranorex(self):
        """Return session profile obj with URI=application://com.fnfr.itest.applications.ranorex 
        
           Ranorex Test session provides ways of automating the Windows, Web, and Mobile UI applications. The seamless 
           testing of these iTest functionalities is achieved by iTest integration with Ranorex application (www.ranorex.com).
        """
        return SessionProfile(self._protosock, self._agent_type, 'application://com.fnfr.itest.applications.ranorex')
    
    @property
    def selenium(self):
        """Return session profile obj with URI=application://com.fnfr.itest.applications.selenium
        
           For Selenium sessions, iTest opens an instance of the Firefox browser. You interact with the pages in the normal
           way while iTest captures your actions and responses from the session.
        """
        return SessionProfile(self._protosock, self._agent_type, 'application://com.fnfr.itest.applications.selenium')
    
    @property
    def netconf(self):
        """Return session profile obj with URI=application://com.fnfr.itest.applications.spirent.netconf.tool 
        
           The NetConf session window displays your commands and the device's responses. You can think of the session window
           as aterminal - a terminal to a NetConf service as a subsystem that iTest is monitoring and capturing.        
        """
        return SessionProfile(self._protosock, self._agent_type, 'application://com.fnfr.itest.applications.spirent.netconf.tool')
    
    @property
    def posapp(self):
        """Return session profile obj with URI=application://com.fnfr.itest.applications.spirent.posapp.tool """
        return SessionProfile(self._protosock, self._agent_type, 'application://com.fnfr.itest.applications.spirent.posapp.tool')
    
    @property
    def udp(self):
        """Return session profile obj with URI=application://com.fnfr.itest.applications.udp.tool
        
        For UDP sessions, the computer running iTest communicates directly over UDP (User Datagram Protocol) with the specified device.
        For each open session, the UDP session window displays your commands and the local echo. Open another session to view the device's
        response. You can think of the window as a terminal client - a terminal that iTest is monitoring and capturing.    
        """
        return SessionProfile(self._protosock, self._agent_type, 'application://com.fnfr.itest.applications.udp.tool')
    
    @property
    def vmware_core(self):
        """Return session profile obj with URI=application://com.fnfr.itest.applications.vmware.vi.core.tool """
        return SessionProfile(self._protosock, self._agent_type, 'application://com.fnfr.itest.applications.vmware.vi.core.tool')
    
    @property
    def vmware_vscphere(self):
        """Return session profile obj with URI=application://com.fnfr.itest.applications.vmware.vsphere.core.too """
        return SessionProfile(self._protosock, self._agent_type, 'application://com.fnfr.itest.applications.vmware.vsphere.core.tool')
    
    @property
    def vnc(self):
        """Return session profile obj with URI=application://com.fnfr.itest.applications.vnc 
        
           iTest VNC sessions are intended to help you to control a remote OS to perform configuration/setup/tear-down tasks. iTest
           VNC sessions are not intended to enable you to thoroughly test an application on a remote platform.
        """
        return SessionProfile(self._protosock, self._agent_type, 'application://com.fnfr.itest.applications.vnc')
    
    @property
    def webservices(self):
        """Return session profile obj with URI=application://com.fnfr.itest.applications.webservices 
        
           The Web Services session window provides a work surface for composing and submitting Web Service requests. 
           Any request that you submit in the iTest session is forwarded to the Web Services server.
           The Web Service performs the action and returns its normal response.
        """
        return SessionProfile(self._protosock, self._agent_type, 'application://com.fnfr.itest.applications.webservices')
    
    @property
    def rest(self):
        """Return session profile obj with URI=application://com.fnfr.itest.applications.webservices.restful 
        
        The REST session window provides a work surface for composing and submitting HTTP requests. Any request that
        you submit in the iTest session is forwarded to the RESTful service. The service performs the action and returns
        its normal response. iTest captures all of the actions that you perform in a session and all of the responses.
        """
        return SessionProfile(self._protosock, self._agent_type, 'application://com.fnfr.itest.applications.webservices.restful')
    
    @property
    def xmlrpc(self):
        """Return session profile obj with URI=application://com.fnfr.itest.applications.webservices.xmlrpc 
        
          The XML-RPC session window provides a work surface for composing and submitting XML-RPC method calls over HTTP and HTTPS.
          Any request that you submit in the iTest session is forwarded to the XML-RPC service. The service performs the action and 
          returns a response. You can view the response in the Response section of the XML-RPC session window. iTest captures all of
          the actions that you perform in a session and all of the responses. You can use the captured items to create test case 
          steps that interact with the XML-RPC server.     
        """
        return SessionProfile(self._protosock, self._agent_type, 'application://com.fnfr.itest.applications.webservices.xmlrpc')
    
    @property
    def database(self):
        """Return session profile obj with URI=application://com.fnfr.itest.tools.database 
        
          The Database Client session window is an interactive browser where you perform database operations and monitor responses.
          iTest captures all commands and responses and you an save captured items as test case steps that start the session and
          set and request database records. Automated test cases open the same session window to perform database operations.
          iTest supports sessions with MySQL, SqlServer, Oracle, Szlite, Derby, or a custom database type that you specify.        
        """
        return SessionProfile(self._protosock, self._agent_type, 'application://com.fnfr.itest.tools.database')
    
    @property
    def flex(self):
        """Return session profile obj with URI=application://com.fnfr.itest.tools.flex 
        
           iTest supports testing Flash applications that were developed using Adobe Flex. 
           iTest can capture your interactions with Flex applications that are hosted on web pages.
           iTest captures each step of an interactive (manual) test in the Flex application. 
        """
        return SessionProfile(self._protosock, self._agent_type, 'application://com.fnfr.itest.tools.flex')
    
    @property
    def windowsgui(self):
        """Return session profile obj with URI=application://com.fnfr.itest.tools.windowsgui """
        return SessionProfile(self._protosock, self._agent_type, 'application://com.fnfr.itest.tools.windowsgui')
    
  #  @property
  #  def testtool(self):
  #     """Return session profile obj with URI=application://com.fnfr.itest.applications.webservices.xmlrpc """
  #      return SessionProfile(self._protosock, self._agent_type, 'application://com.fnfr.open.automation.tool.testtool')
    
    @property
    def agilent(self):
        """Return session profile obj with URI=application://com.fnfr.svt.applications.agilent """
        return SessionProfile(self._protosock, self._agent_type, 'application://com.fnfr.svt.applications.agilent')
    
    @property
    def http(self):
        """Return session profile obj with URI=application://com.fnfr.svt.applications.http 
           
           Using an HTTP session, a test case can talk directly with a device using the HTTP protocol operations GET and POST.
           HTTP GET commands are useful in cases where you are not testing a Web application, but rather are testing something 
           like a device through which the HTTP is passing. 
        """
        return SessionProfile(self._protosock, self._agent_type, 'application://com.fnfr.svt.applications.http')
    
    @property
    def ixia_ixload(self):
        """Return session profile obj with URI=application://com.fnfr.svt.applications.ixia.ixload
        
        To run an IxLoad session in iTest, you first define a test or tests in IxLoad and save the configuration file (RFX file)
        as you normally would. You then start the IxLoad session in iTest and run the IxLoad test. When the session starts, it checks
        out the libraries and connects and runs the initialization script and then loads the configuration, validates it, and transforms
        it (see the description for the iTest load command). The test produces responses and CSV files, which you can then post-process.
        The IxLoad session window is an interactive terminal where you enter commands to perform IxLoad actions on the device. IxLoad returns
        text responses. The responses to IxLoad responses are structured and iTest uses built-in mappers to supply read-made queries.
        """
        return SessionProfile(self._protosock, self._agent_type, 'application://com.fnfr.svt.applications.ixia.ixload')
    
    @property
    def ixia_ixnetwork(self):
        """Return session profile obj with URI=application://com.fnfr.svt.applications.ixia.ixnetwork 
        
        In IxNetwork sessions, you can start and stop traffic, start and stop capture, and review statistics.
        The levels of the Object Data Matrix are represented as subdirectories (see Table 1-11, API Command Data 
        ModelStructure in the IxNetwork Tcl API Guide). The iTest IxNetwork session window enables you to navigate
        the object hierarchy by using commands that you typically use to navigate a file system.
        """
        return SessionProfile(self._protosock, self._agent_type, 'application://com.fnfr.svt.applications.ixia.ixnetwork')
    
    @property
    def ixiaTraffic(self):
        """Return session profile obj with URI=application://com.fnfr.svt.applications.ixiaTraffic 
        
        The Ixia Traffic session window is an interactive terminal where you enter commands to perform Ixia Traffic actions 
        on the device. The session on the Ixia device returns text responses.     
        """
        return SessionProfile(self._protosock, self._agent_type, 'application://com.fnfr.svt.applications.ixiaTraffic')
    
 #   @property
 #    def swing(self):
 #       """Return session profile obj with URI=application://com.fnfr.svt.applications.java.swing 
 #          
 #      The Swing session type allows you to test Java applications with user interfaces that were developed using Swing.
 #      """
 #       return SessionProfile(self._protosock, self._agent_type, 'application://com.fnfr.svt.applications.java.swing')
    
    @property
    def smtpmail(self):
        """Return session profile obj with URI=application://com.fnfr.svt.applications.mail 
        
           You can add steps that construct and send email messages during execution. A test case can construct and send as 
           many email messages as are needed. The message body can contain both fixed text and test response and result data.
           """
        return SessionProfile(self._protosock, self._agent_type, 'application://com.fnfr.svt.applications.mail')
    
    @property
    def process(self):
        """Return session profile obj with URI=application://com.fnfr.svt.applications.process
        
           Execute and manage processes on the computer that is running iTest. 
        """
        return SessionProfile(self._protosock, self._agent_type, 'application://com.fnfr.svt.applications.process')
    
    @property
    def python(self):
        """Return session profile obj with URI=application://com.fnfr.svt.applications.python 
          
          iTest Python session is a terminal session, similar to Tcl Shell. The session uses native Python interpreter
          to getting responses. Supported versions of interpreters are: 2.4-2.7 and 3.0-3.6. iTest support an internal 
          Python interpreter and also allows you to point to an external interpreter via Preferences settings.       
        """
        return SessionProfile(self._protosock, self._agent_type, 'application://com.fnfr.svt.applications.python')
    
    @property
    def smartbits(self):
        """Return session profile obj with URI=application://com.fnfr.svt.applications.smartbits 
        
           The SmartBits session window is an interactive terminal where you enter commands to perform 
           SmartBits actions on the device. SmartBits returns text responses.
        """
        return SessionProfile(self._protosock, self._agent_type, 'application://com.fnfr.svt.applications.smartbits')
    
    @property
    def snmp(self):
        """Return session profile obj with URI=application://com.fnfr.svt.applications.snmp 
        
           A hierarchical browser for getting and setting SNMP MIB data using the Simple Network Management Protocol
           (SNMP V1, V2c, or V3) defined in RFC 1157
        """
        return SessionProfile(self._protosock, self._agent_type, 'application://com.fnfr.svt.applications.snmp')
    
    @property
    def avalanche(self):
        """Return session profile obj with URI=application://com.fnfr.svt.applications.spirent.avalanche 
        
           There are several options for running an Avalanche test in iTest:
			* Running a test using the Tcl scripts generated by Avalanche
			* Running an Avalanche test on a TestCenter device
			* Running a test using Demo mode 
        """
        return SessionProfile(self._protosock, self._agent_type, 'application://com.fnfr.svt.applications.spirent.avalanche')
    
    @property
    def testcenter_gui(self):
        """Return session profile obj with URI=application://com.fnfr.svt.applications.spirent.testcenter.gui """
        return SessionProfile(self._protosock, self._agent_type, 'application://com.fnfr.svt.applications.spirent.testcenter.gui')
    
    @property
    def spirenttestcenter(self):
        """Return session profile obj with URI=application://com.fnfr.svt.applications.spirenttestcenter
        
           iTest integrates with Spirent TestCenter (STC) and provides REST API to ensure that the automation functionality is readily available
           to a wide variety of clients using both script and GUI. This integration also eliminates the requirement of an STC installation on the 
           client system. This allows you to use existing tools available to create STC automation clients that can run on any platform.    
        """
        return SessionProfile(self._protosock, self._agent_type, 'application://com.fnfr.svt.applications.spirenttestcenter')

    @property
    def ssh(self):
        """Return session profile obj with URI=application://com.fnfr.svt.applications.ssh 
           
           A virtual terminal that communicates with a device using the SSH protocol (SSH-1 or SSH-2) defined in RFC 4250
           Note: Because Linux and Unix devices typically include Telnet and SSH servers, you can start a Telnet or SSH session
           to "localhost" to access a shell.
        """
        return SessionProfile(self._protosock, self._agent_type, 'application://com.fnfr.svt.applications.ssh')

    @property
    def syslog(self):
        """Return session profile obj with URI=application://com.fnfr.svt.applications.syslog
        
           Each Syslog session monitors the syslog messages that arrive at the built-in iTest syslog server (visible in the Syslog view).
           While the syslog server receives all messages, any syslog session can filter the messages based on the following property settings 
           in the session profile.
           As a result of configuring session profile settings, only the messages that meet the filter settings appear in the session window.
           This enables your test cases to analyze the particular responses (messages) of interest and to ignore irrelevant messages.     
        """
        return SessionProfile(self._protosock, self._agent_type, 'application://com.fnfr.svt.applications.syslog')

    @property
    def tclsh(self):
        """Return session profile obj with URI=application://com.fnfr.svt.applications.tclsh 
        
           You can type Tcl commands and expressions into the iTest Tcl Shell session window.
           iTest captures your commands and the interpreter's responses.
        """
        return SessionProfile(self._protosock, self._agent_type, 'application://com.fnfr.svt.applications.tclsh')

    @property
    def telnet(self):
        """Return session profile obj with URI=application://com.fnfr.svt.applications.telnet 
        
           A virtual terminal that communicates with a device using the Telnet protocol defined in RFC 854
           Note: Because Linux and Unix devices typically include Telnet and SSH servers, you can start a Telnet 
           or SSH session to "localhost" to access a shell.      
        """
        return SessionProfile(self._protosock, self._agent_type, 'application://com.fnfr.svt.applications.telnet')

    @property
    def web(self):
        """Return session profile obj with URI=application://com.fnfr.svt.applications.web """
        return SessionProfile(self._protosock, self._agent_type, 'application://com.fnfr.svt.applications.web')

    @property
    def powershell(self):
        """Return session profile obj with URI=application://com.spirent.itest.applications.powershell

           PowerShell sessions execute commands at the Windows PowerShell prompt. The PowerShell session 
           window displays your commands and the local PC's responses.
        """
        return SessionProfile(self._protosock, self._agent_type, 'application://com.spirent.itest.applications.powershell')

    @property
    def wireshark(self):
        """Return session profile obj with URI=application://com.fnfr.svt.applications.wireshark 
        
           Wireshark sessions provide a command line interface for interactively capturing packets from a network interface.
           For commands that return status and packet data, iTest saves the responses as structured data and generates
           associated queries to simplify pass/fail analysis. 
        """
        return SessionProfile(self._protosock, self._agent_type, 'application://com.fnfr.svt.applications.wireshark')

    @property
    def landsliderest(self):
        """Return session profile obj with URI=application://com.spirent.itest.applications.landsliderest 

        You can use iTest to automate Spirent Landslide tests.
        In iTest, when you start a Landslide session, iTest launches the Landslide TAS user interface running on the Landslide device.
          1. Now you can interact with the TAS in the normal way. For example, you might load a test configuration, start the test session,
        collect the responses, wait to collect several data sets, stop the test session, request the test session results, and then close the test session.
          2. When you are finished working on the TAS, you return to iTest and close the Landslide session window.
          3. You can now save the captured steps and responses as an iTest test case. As needed, you can modify and update the test case (for example, modify
        the ImportTestSuite step by causing it to load a different Landslide test suite file or replace the filename with a variable whose value is set at runtime). 
        """
        return SessionProfile(self._protosock, self._agent_type, 'application://com.spirent.itest.applications.landsliderest')

    @property
    def popmail(self):
        """Return session profile obj with URI=application://com.spirent.itest.applications.popmail 
        
           You may se the Mail (POP3) sessions to retrieve emails from subscribers, view content, extract the required text and
           attachments, save the attached images as individual files, and then insert them into other sessions, for example, Selenium.
        """
        return SessionProfile(self._protosock, self._agent_type, 'application://com.spirent.itest.applications.popmail')

#    @property
#   def scriptingSession(self):
#        """Return session profile obj with URI=application://com.spirent.itest.applications.sf.scriptingSession 
# 
#        The Script Library Support session type enables your iTest test cases to invoke functions and procedures
#        that are defined in external script libraries (your existing "home-grown" procedure libraries).
#       """
#        return SessionProfile(self._protosock, self._agent_type, 'application://com.spirent.itest.applications.sf.scriptingSession')

    @property
    def stcrest(self):
        """Return session profile obj with URI=application://com.spirent.itest.applications.stcrest """
        return  SessionProfile(self._protosock, self._agent_type, 'application://com.spirent.itest.applications.stcrest')

    @property
    def eggplant(self):
        """Return session profile obj with URI=application://com.spirent.itest.applications.testplant
        
           EggPlant Test session provides ways of automating execution of image-based GUI testing. The seamless testing of these iTest functionalities
           is achieved by iTest integration with EggPlant application (http://www.testplant.com/eggplant/testing-tools/eggplant-developer/).
        """
        return  SessionProfile(self._protosock, self._agent_type, 'application://com.spirent.itest.applications.testplant')

